;;; -*-emacs-lisp-*-
;;; ====================================================================
;;;  @Emacs-Lisp-file{
;;;     author          = "Nelson H. F. Beebe",
;;;     version         = "1.00",
;;;     date            = "08 December 2001",
;;;     time            = "14:32:29 MST",
;;;     filename        = "hoc.el",
;;;     address         = "Center for Scientific Computing
;;;                        University of Utah
;;;                        Department of Mathematics, 322 INSCC
;;;                        155 S 1400 E RM 233
;;;                        Salt Lake City, UT 84112-0090
;;;                        USA",
;;;     telephone       = "+1 801 581 5254",
;;;     FAX             = "+1 801 585 1640, +1 801 581 4148",
;;;     URL             = "http://www.math.utah.edu/~beebe",
;;;     checksum        = "35147 93 367 3682",
;;;     email           = "beebe@math.utah.edu, beebe@acm.org,
;;;                        beebe@computer.org, beebe@ieee.org
;;;                        (Internet)",
;;;     codetable       = "ISO/ASCII",
;;;     keywords        = "hoc (high-order calculator)",
;;;     supported       = "yes",
;;;     docstring       = "This is an Emacs Lisp function that augments
;;;                        c-mode to make a simple hoc-mode.
;;;
;;;                        The checksum field above contains a CRC-16
;;;                        checksum as the first value, followed by the
;;;                        equivalent of the standard UNIX wc (word
;;;                        count) utility output of lines, words, and
;;;                        characters.  This is produced by Robert
;;;                        Solovay's checksum utility.",
;;;  }
;;; ====================================================================

(defun hoc-mode ()
  "Set up for editing hoc code (a minor variation on C mode)."
  (interactive)
  (let ((empty-buffer (= (point-min) (point-max))))
    (goto-char (point-min)) ;prevent c-mode's insertion of C template file
    (if empty-buffer
        (insert "#!\n"))
    (c-mode)
    (make-local-variable 'comment-start)
    (setq comment-start "# ")
    (make-local-variable 'comment-end)
    (setq comment-end "")
    (make-local-variable 'comment-column)
    (make-local-variable 'comment-start-skip)
    (setq comment-start-skip "# *")
    (setq mode-name "hoc")
    (setq c-argdecl-indent 0
	  c-auto-newline t
	  c-basic-offset 8
	  c-brace-imaginary-offset 0
	  c-brace-offset 0
	  c-continued-statement-offset 0
	  c-indent-level 8
	  c-label-offset -8
	  case-fold-search nil	      ;case sensitive searching in hoc
	  comment-column 40)
    (goto-char (point-min)) ;prevent c-mode's possible insertion of C template file
    (if (and empty-buffer (looking-at "^#!\n"))
	(delete-char 3))
    (run-hooks 'hoc-mode-hook)
    (goto-char (point-min))
    (if empty-buffer
	(progn
	  (insert "### -*-hoc-*-\n")
	  (insert "### ====================================================================\n")
	  (insert "### \n")
	  (insert "### ")
	  (date)
	  (insert "\n")
	  (insert "### ====================================================================\n")
	  (goto-char (point-min))
	  (forward-line 2)
	  (forward-char 4)
	  (set-buffer-modified-p nil)))))

(if (string-lessp (substring emacs-version 0 2) "19") ; earlier than 19.x
    (progn
      (setq auto-mode-alist
	    (cons (cons "\\.hoc$" 'hoc-mode) auto-mode-alist))
      (autoload 'hoc-mode  "hoc"
	"Enter hoc mode." t nil))
  (progn
    (if (not (assoc "\\.hoc$" auto-mode-alist))
	(setq auto-mode-alist
	      (cons (cons "\\.hoc$" 'hoc-mode) auto-mode-alist)))
    (autoload 'hoc-mode  "hoc"
      "Enter hoc (high-order calculator) mode." t nil)))
